/*
 * Copyright 2010 James Pether Sörling
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *	$Id: ApplicationManagerImpl.java 5442 2011-04-26 19:16:26Z pether $
 *  $HeadURL: https://cia.svn.sourceforge.net/svnroot/cia/tags/parent-pom-2011.05.31/service.impl/src/main/java/com/hack23/cia/service/impl/ApplicationManagerImpl.java $
*/

package com.hack23.cia.service.impl;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import com.hack23.cia.model.internal.application.data.impl.AggregatedBugData;
import com.hack23.cia.model.internal.application.data.impl.AggregatedCountryData;
import com.hack23.cia.service.api.AgentContainer;
import com.hack23.cia.service.api.ApplicationManager;
import com.hack23.cia.service.api.DataContainer;
import com.hack23.cia.service.api.DataSummary;
import com.hack23.cia.service.api.DataSummary.DataSummaryId;
import com.hack23.cia.service.api.GeoIpCountryService;
import com.hack23.cia.service.api.ServiceRequest;
import com.hack23.cia.service.api.ServiceResponse;
import com.hack23.cia.service.api.Translator;
import com.hack23.cia.service.data.api.OperationalStatementDAO;
import com.hack23.cia.service.impl.common.BusinessService;

/**
 * The Class ApplicationManagerImpl.
 */
@Service(value="ApplicationManager")
public final class ApplicationManagerImpl implements ApplicationManager {

	/** The aggregated bug data data container. */
	@Autowired
	@Qualifier("AggregatedBugDataContainer")
	private DataContainer<AggregatedBugData,String> aggregatedBugDataDataContainer;

	/** The operational statement dao. */
	@Autowired
	@Qualifier("OperationalStatementDAO")
	private OperationalStatementDAO operationalStatementDAO;

	/** The aggregated country data data container. */
	@Autowired
	@Qualifier("AggregatedCountryDataContainer")
	private DataContainer<AggregatedCountryData,String> aggregatedCountryDataDataContainer;

	/** The authentication manager. */
	@Autowired
	@Qualifier("org.springframework.security.authenticationManager")
	private AuthenticationManager authenticationManager;

	/** The data agent container. */
	@Autowired
	@Qualifier("DataAgentContainer")
	private DataAgentContainerImpl dataAgentContainer;

	/** The data summary data container. */
	@Autowired
	@Qualifier("DataSummaryDataContainer")
	private DataContainer<DataSummary,DataSummaryId> dataSummaryDataContainer;


	/** The service request business service map. */
	private final Map<Class<? extends ServiceRequest>, BusinessService> serviceRequestBusinessServiceMap = new HashMap<Class<? extends ServiceRequest>, BusinessService>();

	/** The translator. */
	@Autowired
	@Qualifier("Translator")
	private Translator translator;

	/** The geo ip country service. */
	@Autowired
	@Qualifier("GeoIpCountryService")
	private GeoIpCountryService geoIpCountryService;

	/**
	 * Instantiates a new application manager impl.
	 */
	public ApplicationManagerImpl() {
		super();
		// TODO Auto-generated constructor stub
	}

	/**
	 * Instantiates a new application manager impl.
	 *
	 * @param businessServices
	 *            the business services
	 */
	public ApplicationManagerImpl(final List<BusinessService> businessServices) {
		super();
		for (final BusinessService serviceHandler : businessServices) {
			serviceRequestBusinessServiceMap.put(serviceHandler
					.getSupportedService(), serviceHandler);
		}
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.api.ApplicationManager#authenticate(java.lang.String, java.lang.String)
	 */
	@Override
	public Object authenticate(final String username, final String password) {
		final Authentication auth = new UsernamePasswordAuthenticationToken(
				username, password);

		final Authentication returned = authenticationManager.authenticate(auth);
		SecurityContextHolder.getContext().setAuthentication(
				returned);

		return returned;
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.api.ApplicationManager#getAgentContainer(java.io.Serializable)
	 */
	@Override
	public AgentContainer getAgentContainer() {
		return dataAgentContainer;
	}


	/* (non-Javadoc)
	 * @see com.hack23.cia.service.api.ApplicationManager#getDataContainer(java.io.Serializable)
	 */
	@Override
	public  <T extends Serializable,ID extends Serializable> DataContainer<T,ID> getDataContainer(
			final Class<T> dataObject) {

		if (dataObject.equals(DataSummary.class)) {
			return (DataContainer<T,ID>) dataSummaryDataContainer;
		} else if (dataObject.equals(AggregatedCountryData.class)) {
			return (DataContainer<T,ID>) aggregatedCountryDataDataContainer;
		} else if (dataObject.equals(AggregatedBugData.class)) {
			return (DataContainer<T,ID>) aggregatedBugDataDataContainer;
		} else 	{
			return null;
		}
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.api.ApplicationManager#getGeoIpCountryService()
	 */
	@Override
	public GeoIpCountryService getGeoIpCountryService() {
		return geoIpCountryService;
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.api.ApplicationManager#getOperationalStatementDAO()
	 */
	@Override
	public OperationalStatementDAO getOperationalStatementDAO() {
		return operationalStatementDAO;
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.api.ApplicationManager#getTranslator()
	 */
	@Override
	public Translator getTranslator() {
		return translator;
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.api.ApplicationManager#logout()
	 */
	@Override
	public void logout() {
		SecurityContextHolder.getContext().setAuthentication(null);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.ApplicationManager#service(com.hack23.cia.service
	 * .api.ServiceRequest)
	 */
	@Override
	@Secured( {"ROLE_ANONYMOUS"})
	public final ServiceResponse service(final ServiceRequest serviceRequest) {
		final BusinessService businessService = serviceRequestBusinessServiceMap
				.get(serviceRequest.getClass());
		ServiceResponse serviceResponse = null;

		if (businessService != null) {
			serviceResponse = businessService.processService(serviceRequest);
		}

		return serviceResponse;
	}

}
