/*
 * Copyright 2010 James Pether Sörling
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *	$Id: GeoIpCountryServiceImpl.java 5447 2011-04-26 20:14:31Z pether $
 *  $HeadURL: https://cia.svn.sourceforge.net/svnroot/cia/tags/parent-pom-2011.05.31/service.impl/src/main/java/com/hack23/cia/service/impl/GeoIpCountryServiceImpl.java $
*/
package com.hack23.cia.service.impl;

import java.io.File;
import java.io.IOException;

import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.hack23.cia.service.api.GeoIpCountryService;
import com.hack23.cia.service.api.SystemConfiguration;
import com.maxmind.geoip.Country;
import com.maxmind.geoip.Location;
import com.maxmind.geoip.LookupService;

/**
 * The Class GeoIpCountryServiceImpl.
 */
@Component(value = "GeoIpCountryService")
public final class GeoIpCountryServiceImpl implements GeoIpCountryService {

	/**
	 * The Class IpLocationImpl.
	 */
	private static class IpLocationImpl implements IpLocation {

		/** The country code. */
		public String countryCode;

		/** The country name. */
		public String countryName;

		/** The region. */
		public String region;

		/** The city. */
		public String city;

		/** The postal code. */
		public String postalCode;

		/** The latitude. */
		public float latitude;

		/** The longitude. */
		public float longitude;

		/**
		 * Instantiates a new ip location impl.
		 *
		 * @param location
		 *            the location
		 */
		public IpLocationImpl(final Location location) {
			countryCode = location.countryCode;
			countryName = location.countryName;
			region = location.region;
			city = location.city;
			postalCode = location.postalCode;
			latitude = location.latitude;
			longitude = location.longitude;
		}

		/**
		 * Instantiates a new ip location impl.
		 *
		 * @param countryCode
		 *            the country code
		 * @param countryName
		 *            the country name
		 * @param region
		 *            the region
		 * @param city
		 *            the city
		 * @param postalCode
		 *            the postal code
		 * @param latitude
		 *            the latitude
		 * @param longitude
		 *            the longitude
		 */
		public IpLocationImpl(final String countryCode, final String countryName,
				final String region, final String city, final String postalCode, final float latitude,
				final float longitude) {
			super();
			this.countryCode = countryCode;
			this.countryName = countryName;
			this.region = region;
			this.city = city;
			this.postalCode = postalCode;
			this.latitude = latitude;
			this.longitude = longitude;
		}

		/* (non-Javadoc)
		 * @see com.hack23.cia.service.api.GeoIpCountryService.IpLocation#getCity()
		 */
		@Override
		public String getCity() {
			return city;
		}

		/* (non-Javadoc)
		 * @see com.hack23.cia.service.api.GeoIpCountryService.IpLocation#getCountryCode()
		 */
		@Override
		public String getCountryCode() {
			return countryCode;
		}

		/* (non-Javadoc)
		 * @see com.hack23.cia.service.api.GeoIpCountryService.IpLocation#getCountryName()
		 */
		@Override
		public String getCountryName() {
			return countryName;
		}

		/* (non-Javadoc)
		 * @see com.hack23.cia.service.api.GeoIpCountryService.IpLocation#getLatitude()
		 */
		@Override
		public float getLatitude() {
			return latitude;
		}

		/* (non-Javadoc)
		 * @see com.hack23.cia.service.api.GeoIpCountryService.IpLocation#getLongitude()
		 */
		@Override
		public float getLongitude() {
			return longitude;
		}

		/* (non-Javadoc)
		 * @see com.hack23.cia.service.api.GeoIpCountryService.IpLocation#getPostalCode()
		 */
		@Override
		public String getPostalCode() {
			return postalCode;
		}

		/* (non-Javadoc)
		 * @see com.hack23.cia.service.api.GeoIpCountryService.IpLocation#getRegion()
		 */
		@Override
		public String getRegion() {
			return region;
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() {
			return "IpLocationImpl [countryCode=" + countryCode
					+ ", countryName=" + countryName + ", region=" + region
					+ ", city=" + city + ", postalCode=" + postalCode
					+ ", latitude=" + latitude + ", longitude=" + longitude
					+ "]";
		}
	}

	/** The Constant LOGGER. */
	private final static Logger LOGGER = LoggerFactory
			.getLogger(GeoIpCountryServiceImpl.class);

	/** The country service. */
	private LookupService countryService;

	/** The city service. */
	private LookupService cityService;

	/** The configuration. */
	@Autowired
	private SystemConfiguration configuration;

	/**
	 * Instantiates a new geo ip country service impl.
	 */
	public GeoIpCountryServiceImpl() {
		super();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.api.GeoIpCountryService#getCountryCode(java.lang
	 * .String)
	 */
	@Override
	public String getCountryCode(final String ipAddress) {
		if (countryService != null) {
			final Country country = countryService.getCountry(ipAddress);
			if (country != null) {
				return country.getCode();
			}
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.api.GeoIpCountryService#getLocation(java.lang.String)
	 */
	@Override
	public IpLocation getLocation(final String ipAddress) {
		if (cityService != null) {
			final Location location = cityService.getLocation(ipAddress);
			if (location != null) {

				final IpLocationImpl ipLocationImpl = new IpLocationImpl(location);
				LOGGER.info(ipLocationImpl.toString());
				return ipLocationImpl;
			}
		}
		return null;
	}

	/**
	 * Inits the lookup service.
	 */
	@PostConstruct
	public void initLookupService() {
		try {
			countryService = new LookupService(new File(
					configuration.getGeoipDataFileLocation()));
		} catch (final IOException e) {
			LOGGER.warn("Add msg",e);
		}
		try {
			cityService = new LookupService(new File(
					configuration.getGeoipDataFileLocationCity()));
		} catch (final IOException e) {
			LOGGER.warn("Add msg",e);
		}
	}

}
