/*
 * Copyright 2010 James Pether Sörling
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *	$Id: TranslatorServiceImpl.java 5442 2011-04-26 19:16:26Z pether $
 *  $HeadURL: https://cia.svn.sourceforge.net/svnroot/cia/tags/parent-pom-2011.05.31/service.impl/src/main/java/com/hack23/cia/service/impl/TranslatorServiceImpl.java $
 */
package com.hack23.cia.service.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.api.translate.Language;
import com.google.api.translate.Translate;
import com.hack23.cia.model.internal.application.system.impl.LanguageContentData;
import com.hack23.cia.model.internal.application.system.impl.LanguageContentType;
import com.hack23.cia.service.data.api.LanguageContentDataDAO;

/**
 * The Class TranslatorServiceImpl.
 */
@Service(value = "TranslatorService")
public final class TranslatorServiceImpl implements TranslatorService {

	/** The Constant LOGGER. */
	private final static Logger LOGGER = LoggerFactory
			.getLogger(TranslatorServiceImpl.class);

	/** The application language content data dao. */
	@Autowired
	private LanguageContentDataDAO applicationLanguageContentDataDAO;

	/** The language map. */
	private final Map<String, LanguageContentData> languageMap = new HashMap<String, LanguageContentData>();

	/**
	 * Call translate.
	 *
	 * @param text
	 *            the text
	 * @param from
	 *            the from
	 * @param to
	 *            the to
	 * @return the string
	 */
	private String callTranslate(final String text, final Language from,
			final Language to) {
		try {
			Translate.setHttpReferrer("http://cia.sourceforge.net/");
			LOGGER.info("Call google translate:" + text);
			return Translate.execute(text, from, to);
		} catch (final Exception e) {
			LOGGER.warn("Problem with google translate", e);
		}
		return text;
	}

	/**
	 * Find translation.
	 *
	 * @param key
	 *            the key
	 * @param from
	 *            the from
	 * @param to
	 *            the to
	 * @return the language content data
	 */
	private LanguageContentData findTranslation(final String key,
			final String from, final String to) {
		LanguageContentData languageContentData = languageMap.get(key + "."
				+ from + "." + to);
		if (languageContentData == null) {
			languageContentData = applicationLanguageContentDataDAO
					.findTranslation(key, from, to);
		}

		return languageContentData;
	}

	/**
	 * Gets the save translation.
	 *
	 * @param text
	 *            the text
	 * @param from
	 *            the from
	 * @param to
	 *            the to
	 * @return the save translation
	 */
	private LanguageContentData getSaveTranslation(final String text,
			final Language from, final Language to) {
		LanguageContentData translation;
		final String translatedValue = callTranslate(text, from, to);
		translation = new LanguageContentData();
		translation.setKey(text);
		translation.setFromLanguage(from.toString());
		translation.setToLanguage(to.toString());
		translation.setValue(translatedValue);
		translation.setCreatedDate(new Date());
		translation.setType(LanguageContentType.GOOGLE_TRANSLATION);
		// applicationLanguageContentDataDAO.persist(translation);
		languageMap.put(
				translation.getKey() + "." + translation.getFromLanguage()
						+ "." + translation.getToLanguage(), translation);
		return translation;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.impl.TranslatorService#getTranslation(java.lang
	 * .String, com.google.api.translate.Language,
	 * com.google.api.translate.Language)
	 */
	@Override
	@Transactional
	public String getTranslation(final String text, final Language from,
			final Language to) {
		LanguageContentData translation = findTranslation(text,
				from.toString(), to.toString());

		if (translation == null) {

			// Direct translation, assume google handles English best..
			if (!to.equals(Language.ENGLISH)) {
				LanguageContentData englishTranslation = findTranslation(text,
						from.toString(), Language.ENGLISH.toString());

				if (englishTranslation == null) {
					englishTranslation = getSaveTranslation(text, from,
							Language.ENGLISH);
				}
				translation = getSaveTranslation(englishTranslation.getValue(),
						Language.ENGLISH, to);
			} else {
				translation = getSaveTranslation(text, from, to);
			}

			LOGGER.info("Persist new translation: " + text);
		}
		return translation.getValue();
	}

	/**
	 * Inits the language.
	 */
	@PostConstruct
	void initLanguage() {
		final List<LanguageContentData> all = applicationLanguageContentDataDAO
				.getAll();

		// Check and delete google translations older than 15 days.

		for (final LanguageContentData data : all) {
			languageMap.put(data.getKey() + "." + data.getFromLanguage() + "."
					+ data.getToLanguage(), data);
		}
	}

}
